@@ -1,11 +1,14 @@
 import json
-from .SFAPI import Sobjects, query,restClient,jsonFile,utils,objectUtil,debugLogs,digitalCommerceUtil
+from .SFAPI import Sobjects, query,restClient,jsonFile,utils,objectUtil,debugLogs,digitalCommerceUtil,VlocityErrorLog,ip_attachments,VlocityTrackingEntry
 import logging,time
 import simplejson,sys
 from operator import itemgetter
 from inspect import getmembers, isfunction
 import traceback
 import colorama
+from difflib import SequenceMatcher
+
+#/Users/uormaechea/Documents/Dev/python/InCliLib/InCLipkg/incli/logs/07L3O00000E6dDKUAZ.log
 
 varsFile = 'confIcli.json'
 vars = None
@@ -44,6 +47,34 @@ def listEnvs():
     env = readVars('environment')
     print(f"Current Environment is {env}")
 
+def _bestMatch(value,vlist):
+
+    matches = []
+    for v in vlist:
+        s = SequenceMatcher(None, v, value)
+      #  print(f"{v} {value} {s.ratio()}")
+        if s.ratio() > 0.6:
+            matches.append(v)
+   # print(matches)
+    return matches
+
+def check_modifiers(command,args,modifiers,help=None):
+    ars = [c for c in args if c.startswith('-')]
+    ars.remove(command)
+    if '-u' in ars: ars.remove('-u')
+    if '-debug' in ars: ars.remove('-debug')
+
+    for ar in ars:
+        if ar not in modifiers:
+            matches = _bestMatch(ar,modifiers)
+            mstr = f" Did you mean {utils.CYELLOW}{' or '.join(matches)}{utils.CEND} ?" if len(matches)>0 else ''
+            print(f"The modifier {utils.CYELLOW}{ar}{utils.CEND} is not valid for command {command}. {mstr}")
+            print(f"Valid modifiers are {utils.CGREEN}{' '.join(modifiers)}{utils.CEND}")
+            if help != None:
+                print(help)
+            return False
+    return True
+
 def _queryAndPrint(q,fields=None,systemFields=True,nullFields=False):
 
     res = query.queryRecords(q)
@@ -66,10 +97,14 @@ def option_readme(args):
     if '-hu' in args:
         return
     
+    colorama.just_fix_windows_console()
+
     print(f"""
     {utils.CYELLOW}Significant updates:{utils.CEND}
+     - Changes in version 0.0.61
+       Added {utils.CYELLOW}InCli -ipe{utils.CEND} to allow parsing the Integration Procedure Errors.
 
-     - Changes in version 0.49.0
+     - Changes in version 0.0.49
        Added {utils.CYELLOW}-auto{utils.CEND} to the log parsing. This options activates the debug logs automatically for the -loguser especified. If no -loguser is specified it will do it for the user used to authenticate with the org.  
             {utils.CGREEN}InCli -logs -u orgAlias -tail -loguser username:onboarding@nosdti-parceiros.cs109.force.com -deletelogs -auto.{utils.CEND}
             This command will start listening for new logs for user onboarding@nosdti-parceiros.cs109.force.com in DTI and parsing them in real time. 
@@ -116,20 +151,143 @@ def option_readme(args):
 
     """)
 
-def option_q(args):
+def option_vte(args):
+    help = f"""
+        {utils.CYELLOW}-vte{utils.CEND}    Lists the latest Vlocity Tracking Entries.   {utils.CGREEN}InCli -u orgAlias -vte {utils.CEND}
+            {utils.CYELLOW}-limit number{utils.CEND} The number of records to display
+        {utils.CYELLOW}-vte a6R7a000001k7t7EAA {utils.CEND} - Displays the JSON for the provided Id.
+            {utils.CYELLOW}-file{utils.CEND} Saves the JSON to a file. """ 
+    
+    if '-hu' in args: 
+        return help
+
+    if '-h' in args: return
+
+    if check_modifiers('-vte',args,["-file","-limit"],help) == False: return
+
+    connectionInit(args)
+    id = getParam(args,'-vte')
+    tofile = True if '-file' in args else False
+    limit = getParam(args,'-limit')
+
+    if id != None:
+        return VlocityTrackingEntry.query_and_print_error_record(id=id,tofile=tofile)
+    limit = 100 if limit == None else limit
+    VlocityTrackingEntry.print_error_list(limit=limit)
+    
+def option_ipa(args):
+    help = f"""
+        {utils.CYELLOW}-ipa{utils.CEND}    Lists the latest IP attachments.   {utils.CGREEN}InCli -u orgAlias -ipa {utils.CEND}
+            {utils.CYELLOW}-limit number{utils.CEND} The number of records to display
+
+            {utils.CYELLOW}-around 2023-03-28T16:25:00{utils.CEND} Will display all the attachments around the datetime provided. 10 minutes before and after the datetime provided. 
+            {utils.CYELLOW}-around 2023-03-28T16:25:00 -minutes 25 {utils.CEND} 25 minutes before and after the datetime provided. 
+
+            {utils.CYELLOW}-from 2023-03-28T16:25:00 -to 2023-03-28T18:00:00{utils.CEND} Will display all the attachments between the datetimes provided (from-->to). 
+
+            {utils.CYELLOW}-around 2023-03-28T16:25:00 -contains 8027T000005H16rQAC,"Movel 20GB",coyote23@gmail.com{utils.CEND} Will display all the attachments around the datetime provided containing any of the strings specified (comma separated). 
+            {utils.CYELLOW}-from 2023-03-28T16:25:00 -to 2023-03-28T18:00:00 -contains 8027T000005H16rQAC,"Movel 20GB",coyote23@gmail.com{utils.CEND} Will display all the attachments around the datetime provided containing any of the strings specified. 
+
+            -from","2023-03-09T09:19:31","-to","2023-03-11T13:55:00"
+        {utils.CYELLOW}-ipa a2J7a000002UakuEAC {utils.CEND} - Displays the attachment for the provided Id.
+            {utils.CYELLOW}-file{utils.CEND} Saves the attachment to a file. """ 
+    
+    if '-hu' in args: 
+        return help
+    if '-h' in args: return
+
+    if check_modifiers('-ipa',args,["-file","-limit","-nf","-from","-to","-around","-minutes","-contains","-orderNumber"],help) == False: return
+
+    connectionInit(args)
+    ipa_id = getParam(args,'-ipa')
+    toFile = True if '-file' in args else False
+    limit = getParam(args,'-limit')
+    mfrom = getParam(args,'-from')
+    mto = getParam(args,'-to')
+    maround = getParam(args,'-around')
+    minutes = getParam(args,'-minutes')
+    contains = getParam(args,'-contains')
+    orderNumber = getParam(args,'-orderNumber')
+
+    nf = True if '-nf' in args else False
+
+    if ipa_id != None:
+        return ip_attachments.print_attachments(Id=ipa_id,toFile=toFile)
+
+    if mfrom == None and mto == None and maround==None:    
+        limit = 100 if limit == None else limit
+
+    if orderNumber != None:
+        order = query.query(f"select fields(all) from order where OrderNumber='{orderNumber}' limit 1")
+        account = query.query(f"select fields(all) from account where Id='{order['records'][0]['AccountId']}' limit 1")
+        contains_lst = []
+        contains_lst.append(order['records'][0]['Id'])
+        contains_lst.append(account['records'][0]['vlocity_cmt__PrimaryContactId__c'])
+        contains_lst.append(account['records'][0]['NOS_t_IdConta__c'])
+        contains_lst.append(account['records'][0]['ID_Conta__c'])
+        contains_lst.append(order['records'][0]['OwnerId'])
+
+        contains_lst.append(account['records'][0]['Name'])
+        contains = ','.join(contains_lst)
+
+        maround = order['records'][0]['CreatedDate'].split('.')[0]
+        print(maround)
+        print(contains)
+
+        minutes = 30
+        limit = None
+
+    if maround != None:
+        if minutes != None: minutes = int(minutes)
+        if minutes == None: minutes = 10
+        mfrom,mto = utils.datetimestr_around(maround,minutes=minutes)        
+
+    ip_attachments.print_attachments(limit=limit,only_not_finished=nf,date1=mfrom,date2=mto,contains=contains)
+
+def option_ipe(args):
+    help = f"""
+        {utils.CYELLOW}-ipe{utils.CEND} displays the last 50 errors. 
+            {utils.CYELLOW}-limit number{utils.CEND} The number of records to display
+            {utils.CYELLOW}-last number{utils.CEND} Shows the error Json for the last number of records
+        {utils.CYELLOW}-ipe errorId{utils.CEND} Shows the error JSON for the provided Id"""
+    
     if '-hu' in args:
-        help = f"""
+        return help
+
+    if '-h' in args: return
+
+    if check_modifiers('-ipe',args,["-file","-limit","-last"],help) == False: return
+
+    connectionInit(args)
+    toFile = True if '-file' in args else False
+    limit = getParam(args,'-limit')
+    last = getParam(args,'-last')
+    ipe_id = getParam(args,'-ipe')
+
+    if ipe_id != None:
+        return VlocityErrorLog.query_and_print_error_record(ipe_id,tofile=toFile)
+
+    if last != None:
+        return VlocityErrorLog.print_error_records(last)
+
+    return VlocityErrorLog.print_error_list(limit)
+
+def option_q(args):
+    help = f"""
         {utils.CYELLOW}-q "select..."{utils.CEND} query to execute. System Fields, such as LastModifiedDate, and null fields are not returned. 
             {utils.CYELLOW}-null{utils.CEND} - will print fields with null values
             {utils.CYELLOW}-system{utils.CEND} - will print the system fields
             {utils.CYELLOW}-all{utils.CEND} - will print both nulls and system fields
             {utils.CYELLOW}-fields "a:b:c:..."{utils.CEND} --> print in table format the specified fields.
             {utils.CYELLOW}-fields all{utils.CEND} --> print in table format"""
+    
+    if '-hu' in args:
         return help
 
     if '-h' in args: return
 
-    #else:
+    if check_modifiers('-q',args,["-fields","-null","-system","-all"],help) == False: return
+
     q = getParam(args,'-q')
 
     fields = getParam(args,'-fields') if '-fields' in args else None    
@@ -146,19 +304,19 @@ def option_q(args):
 
 def option_cc(args):
     colorama.just_fix_windows_console()
-
-    if '-hu' in args:
-        help = f"""
+    help = f"""
         {utils.CYELLOW}-cc{utils.CEND}       {utils.CGREEN}InCli -u orgAlias -cc {utils.CEND}
             Checks the catalogs. Gets all catalogs, does a getOffers, getOfferDetails. Performs basketwithoutconfig and basket with config, if -basket is set. 
             {utils.CYELLOW}-code{utils.CEND} - Catalogue Code. does it for a single catalog. 
             {utils.CYELLOW}-list{utils.CEND} - List all the catalogues. 
             {utils.CYELLOW}-basket{utils.CEND} - Performs basket operations. """ 
-
+    if '-hu' in args:
         return help
 
     if '-h' in args: return
 
+    if check_modifiers('-cc',args,["-code","-list","-basket","-account"],help) == False: return
+
     connectionInit(args)
 
    # path = getParam(args,'-checkCatalogs')
@@ -186,6 +344,8 @@ def option_d(args):
 
     if '-h' in args: return
 
+    if check_modifiers('-d',args,[""],help) == False: return
+
     connectionInit(args)
 
     objectField = getParam(args,'-d')
@@ -207,15 +367,19 @@ def option_d(args):
         print(simplejson.dumps(sibbling, indent=4))
 
 def option_l(args):
-    if '-hu' in args:
-        help = f"""
+    help = f"""
         {utils.CYELLOW}-l{utils.CEND} --> current org limits consuptions. """
+    if '-hu' in args:
         return help
+
     if '-h' in args: return
     
+    if check_modifiers('-l',args,[""],help) == False: return
+
     connectionInit(args)
     action = '/services/data/v51.0/limits'
     res = restClient.callAPI(action)
+    restClient.checkError()
     records = []
     for key in res.keys():
         record = {
@@ -236,25 +400,28 @@ def option_l(args):
     utils.printFormated(records)
 
 def option_o(args):
-    if '-hu' in args:
-        help = f"""
+    help = f"""
         {utils.CYELLOW}-o{utils.CEND} --> List all Objects
             {utils.CYELLOW}-name objectName{utils.CEND}  --> get one row from the object
             {utils.CYELLOW}-name objectName:Id{utils.CEND} --> get the row especified by the Id
             {utils.CYELLOW}-like name{utils.CEND} --> where the Name contains the substring name"""
-
+    if '-hu' in args:
         return help
 
     if '-h' in args: return
       
+    if check_modifiers('-o',args,["-like","-count","-name","-limit"],help) == False: return
+
     connectionInit(args)
 
     if '-name' in args:
         obj = getParam(args,'-name')
+        limit = getParam(args,'-limit') if '-limit' in args else None    
+        l = limit if limit != None else 1
         chunks = obj.split(':')
 
         if len(chunks) == 1:
-            q = f"select fields(all) from {chunks[0]} limit 1"
+            q = f"select fields(all) from {chunks[0]} limit {l}"
         else:
             q = f"select fields(all) from {chunks[0]} where Id='{chunks[1]}'"
 
@@ -284,8 +451,6 @@ def option_o(args):
                 print("Quering objects row count.")
                 print("", end=".")          
                 try:
-                    if out['name'] == 'AccountUserTerritory2View':
-                        print()
                     c = query.query(f" select count(Id) from {out['name']}",raiseEx=True)
                     out['count'] = c['records'][0]['expr0']
                 except Exception as e:
@@ -392,6 +557,8 @@ def option_logs(args):
     if '-h' in args:
         return help
 
+    if check_modifiers('-logs',args,["-last","-tail","-store","-deletelogs","-limit","-limitinfo","-auto","-folder","-level","-loguser","-where","-inputfile","-file","-error","-noScreen"],help) == False: return
+
     logId = getParam(args,'-logs')    
     last = getParam(args,'-last')  
     limit = getParam(args,'-limit')    
@@ -400,6 +567,7 @@ def option_logs(args):
     folder = getParam(args,'-folder')    
     is_folder  = True if '-folder' in args else False   
 
+    noScreen  = True if '-noScreen' in args else False
     level = getParam(args,'-level') 
     loguser = getParam(args,'-loguser') 
     whereClause = getParam(args,'-where') 
@@ -418,13 +586,16 @@ def option_logs(args):
         connectionInit(args)
         lim = 50 if limit == None else limit
 
-        if auto:
-            while True:
-                debugLogs.printLogRecords(loguser=loguser,limit=lim,whereClause=whereClause)
-                time.sleep(5)
+        try:
+            if auto:
+                while True:
+                    debugLogs.printLogRecords(loguser=loguser,limit=lim,whereClause=whereClause)
+                    time.sleep(5)
 
-        else:
-            return debugLogs.printLogRecords(loguser=loguser,limit=lim,whereClause=whereClause)
+            else:
+                return debugLogs.printLogRecords(loguser=loguser,limit=lim,whereClause=whereClause)
+        except Exception as e:
+            raise e
 
     parseContext = {
         'logId' :logId,
@@ -443,7 +614,8 @@ def option_logs(args):
         'operation':None,
         'auto':auto,
         'store_logId':store_logId,
-        'search_dir':folder
+        'search_dir':folder,
+        "noScreen":noScreen
     }
 
     if tail:
@@ -463,10 +635,12 @@ def option_logs(args):
     elif logId != None:  #check if already downloaded        
         connectionInit(args)
         return debugLogs.do_parse_logId(parseContext)
+
     else:
         connectionInit(args)
         return debugLogs.do_parse_logs_lastN(parseContext)
 
+
 def option_h(args,type="-h"):
 
     module = __import__('InCli')
@@ -529,10 +703,10 @@ def _main(argsIn):
 
     restClient.glog().debug("In debug mode")
     checkVersion()
-
+    
+    colorama.just_fix_windows_console()
 
     if '-h' in argsIn:
-        colorama.just_fix_windows_console()
 
         help = f"""
 
@@ -566,8 +740,13 @@ def _main(argsIn):
         print()
         print(f" - {utils.CYELLOW}sfdx force:org:list --verbose --all {utils.CEND}--> to list all authorized Orgs and Connection Status")
         print(f" - {utils.CYELLOW}sfdx auth:web:login -r 'Instance Url' -a 'Alias' {utils.CEND}--> to re-authorize")
-        print(f" - {utils.CYELLOW}sfdx auth:web:login -u 'userName'  -a 'alias' {utils.CEND}--> to authorize and Org")
         print()
+        return
+
+    commands = [f"-{fu.split('_')[1]}" for fu in functions]
+    print("Command is not valid.")
+    print(f"Valid commands are {utils.CGREEN}{' '.join(commands)}{utils.CEND}")
+    print(f"Try {utils.CGREEN}InCli -h{utils.CEND} or {utils.CGREEN}InCli -hu{utils.CEND} for more information.")
 
 def main():
     argsIn = sys.argv
