@@ -14,8 +14,6 @@ def getFieldList(objs,name):
     """
     return [obj[name] for obj in objs]
 
-
-
 def Id(obj):
     if type(obj) is str:
         return obj
@@ -67,7 +65,6 @@ def checkId(id,init=None):
             return False
     return re.search(r"\b[a-z0-9]\w{4}0\w{12}|[a-z0-9]\w{4}0\w{9}\b", id)
 
-
 def isId(sobjectType,X):
     if X == None:
         return False
@@ -185,13 +182,12 @@ def get(id,sobjectName=None):
     return query.query(f" select fields(all) from {sobjectName} where Id='{id}' limit 200")
 
 #---------------------------------
-
 def create(sobjectname,object):
     call =  client.callAPI(f'/services/data/{apiVersion}/sobjects/{sobjectname}/', method="post", data=object) 
-    if client.thread_lastCall()['status_code'] == 201:
+    if client.lastCall()['status_code'] == 201:
         logging.info(f"Object {sobjectname} created sucesfully. Id:{call['id']}")
     else:
-        logging.warning(f"Object {sobjectname} creation returned status code {client.thread_lastCall()['status_code']}")
+        logging.warning(f"Object {sobjectname} creation returned status code {client.lastCall()['status_code']}")
         logging.warning(f"Response {call}")
 
         raise ValueError(simplejson.dumps(call, indent=4))
@@ -200,9 +196,9 @@ def create(sobjectname,object):
     return call
 
 def checkError():
-    lc = client.thread_lastCall()
+    lc = client.lastCall()
     if lc['status_code'] >= 400:
-        logging.error(f"Error in call.  statusCode->{client.thread_lastCall()['status_code']}")
+        logging.error(f"Error in call.  statusCode->{client.lastCall()['status_code']}")
         message = {
             'error':lc['error'],
             'errorCode':lc['errorCode']
@@ -258,7 +254,7 @@ def deleteMultiple(sobjectname,id_list):
             raise e
     return call 
 
-def delete(sobjectname,id):
+def delete(sobjectname,id,ts_name=None):
     if id==None:
         return None
     if type(id) is list:
@@ -268,7 +264,7 @@ def delete(sobjectname,id):
     logging.debug(f"deleting {sobjectname} with Id {id}")
     action = f'/services/data/{apiVersion}/sobjects/{sobjectname}/{id}'
     print(action)
-    call =  client.callAPI(action, method="delete")
+    call =  client.callAPI(action, method="delete",ts_name=ts_name)
     try:
         checkError()
     except Exception as e:
