@@ -129,6 +129,9 @@ def do_parse_tail(pc):
                     res = Sobjects.deleteMultiple('ApexLog',l)
                 restClient.glog().debug(f"deleted records {logIds}")
                 delete_queue.task_done()
+            except utils.InCliError as e:
+                if e.args[0]['errorCode'] != 'NO_LOG':
+                    utils.printException(e)
             except Exception as e:
                 print(logIds)
                 print(e)
@@ -176,21 +179,31 @@ def do_parse_tail(pc):
             if len(logRecords) > 0:
                 waitingPrinted = False
 
-                prelogIds = [record['Id'] for record in logRecords]
-                logIds = [id for id in prelogIds if id not in procesed]
-                if len(logIds) == 0:
+                logRecords_not_processed = [r for r in logRecords if r['Id'] not in procesed]
+                ids_not_processed = [r['Id'] for r in logRecords_not_processed]
+                if pc['noScreen']==False:
+                    logIds = [r['Id'] for r in logRecords_not_processed]
+                else:
+                    records1 = []
+                    for r in logRecords_not_processed:
+                        if r['Operation'] not in ['<empty>','VFRemoting']: records1.append(r)
+                        elif r['Operation'] == 'VFRemoting' and r['LogLength']>1000: records1.append(r)
+                    logIds = [r['Id'] for r in records1 ]
+
+                if len(ids_not_processed) == 0:
                     greater = True
                     continue
                 greater = False
-                procesed.extend(logIds)
+                procesed.extend(ids_not_processed)
 
-                parse_apexlogs_by_Ids_or_filepaths(logIds=logIds,pc=pc,raiseKeyBoardInterrupt=True)
+                if len(logIds)>0:
+                    parse_apexlogs_by_Ids_or_filepaths(logIds=logIds,pc=pc,raiseKeyBoardInterrupt=True,raise_no_log=False)
 
                 time0 = logRecords[-1][timefield]
                 timez = time0.split('.')[0] + "Z"
 
                 if delete_queue!=None:
-                    delete_queue.put(logIds)
+                    delete_queue.put(ids_not_processed)
                     restClient.glog().debug(f"{logIds} into queue...")
 
             elif  waitingPrinted == False:
@@ -240,6 +253,8 @@ def do_parse_from_file(parseContext):
     parseContext['operation'] = 'parsefile'
  #   name = os.path.basename(parseContext['filepath']).split('.')[0]
  #   parseContext['logId']=name
+    name = os.path.basename(parseContext['filepath'])
+    parseContext['logId'] = name.split('.')[0]
     context =  parse_apexlog_body(parseContext)
     print_parsed_lines_to_output(parseContext)
     return context
@@ -251,13 +266,19 @@ def do_parse_logId(parseContext):
 
     return context
 
-def parse_apexlogs_by_Ids_or_filepaths(pc,logIds=None,filepaths=None,raiseKeyBoardInterrupt=False,printProgress=False,threads=10,printNum=True):
+def parse_apexlogs_by_Ids_or_filepaths(pc,logIds=None,filepaths=None,raiseKeyBoardInterrupt=False,printProgress=False,threads=10,printNum=True,raise_no_log=True):
     def read_log_from_org(q):
       while True:
-        Id = q.get()
-        get_apexLog_body_from_file_otherwise_db(Id)
-        restClient.glog().debug(f"Read body for Id {Id}")
-        q.task_done()
+        try:
+            Id = q.get()
+            get_apexLog_body_from_file_otherwise_db(Id)
+            restClient.glog().debug(f"Read body for Id {Id}")
+            q.task_done()
+        except Exception as e:
+            if raise_no_log==False and e.args[0]['errorCode'] == 'NO_LOG':
+                pass
+            else:
+                raise e
 
     if 'total_parsed'       not in pc:   pc['total_parsed'] = 0
     if 'parsed_Id_status'   not in pc:   pc['parsed_Id_status'] = []
@@ -268,6 +289,7 @@ def parse_apexlogs_by_Ids_or_filepaths(pc,logIds=None,filepaths=None,raiseKeyBoa
         threads = 0
 
     if threads >0: 
+        threads = 1
         if pc['queue'] == None:
             pc['queue'] = Queue(maxsize=0)
             for x in range(0,threads):
@@ -368,7 +390,6 @@ def get_apexLog_body_from_file_otherwise_db(logId):
         save_to_store(logId,body)
         return logRecord,body,filename
 
-
 def save_to_store(logId,body):
     filename = f"{restClient.logFolder()}{logId}.log"
     file.write(filename,body) 
@@ -484,8 +505,6 @@ def parse_apexlog_body(pc):
         raise e
 
 
-
-
 def print_parsed_lines_to_output(pc):
     def escape_ansi(line):
         ansi_escape =re.compile(r'(\x9B|\x1B\[)[0-?]*[ -\/]*[@-~]')
@@ -558,6 +577,7 @@ def print_parsed_lines(pc):
         if printLimits == False:
             if '*** getCpuTime() ***' in parsedLine['output']:   continue
             if '*** getQueries() ***' in parsedLine['output']:   continue
+            if 'LoggingOpenInterface.' in parsedLine['output']:   continue
             if parsedLine['type'] == 'LIMIT':                    continue
 
         print_parsed_line(pc,parsedLine,num)    
@@ -578,6 +598,7 @@ def isRep(all_loops,parsed_line_index,parsed_line,parsed_lines):
                 parsed_line['loop'] = links
                 parsed_line['is_loop'] = True
                 parsed_line['loop_links'] = links
+                parsed_line['loop_link_lenght'] = link_lenght
                 parsed_line['loop_position'] = parsed_line_index - loop_start_position + 1
 
                 if len(parsed_line['timeStamp']) > 1 and (loop_start_position <= parsed_line_index < loop_start_position + link_lenght):
@@ -612,6 +633,24 @@ def isRep(all_loops,parsed_line_index,parsed_line,parsed_lines):
 
       return False,False
 
+def processRepetition1(parsed_lines):
+
+    for parsed_line in parsed_lines:
+        if 'output' not in parsed_line:  
+            print()
+
+    all_loops = utils.get_all_loops(parsed_lines,"output")
+
+    new_parsed_lines = []
+    for parsedLineNum,parsedLine in enumerate(parsed_lines):
+        if 'apexline' in parsedLine and parsedLine['apexline'] == "161":
+            a=1
+        is_loop, is_first_chain = isRep(all_loops,parsedLineNum,parsedLine,parsed_lines)
+        if is_loop == True and is_first_chain == False:   continue
+        new_parsed_lines.append(parsedLine)
+
+    return new_parsed_lines   
+
 def processRepetition(parsed_lines):
 
     for parsed_line in parsed_lines:
@@ -622,6 +661,9 @@ def processRepetition(parsed_lines):
 
     new_parsed_lines = []
     for parsedLineNum,parsedLine in enumerate(parsed_lines):
+        if 'timeStamp' in parsedLine and parsedLine['timeStamp'][0] == 10188648957:
+            a=1
+            
         is_loop, is_first_chain = isRep(all_loops,parsedLineNum,parsedLine,parsed_lines)
         if is_loop == True and is_first_chain == False:   continue
         new_parsed_lines.append(parsedLine)
@@ -630,7 +672,8 @@ def processRepetition(parsed_lines):
 
 def emptyString(context,size,char=' ',ident=None):
  #   str = ''
-    if ident is None:   ident = context['ident']
+    if ident is None:   
+        ident = context['ident']
     length = ident * size
     str = " "*length
  #   for x in range(length):   str = str + char  
@@ -681,7 +724,7 @@ def print_parsed_line(pc,d,parsed_line_index):
     elif d['type'] == 'DML':    Cinit =  utils.CYELLOW
     elif d['type'] == 'CODE_UNIT':  Cinit =  utils.CYELLOW
 
-    identation = f"{emptyString(context,3,' ',level)}"
+    identation = f"{emptyString(context,1,' ',level)}"
 
     if 'output' not in d:
         print()
@@ -715,6 +758,8 @@ def print_parsed_line(pc,d,parsed_line_index):
 
     wait_time = f"{wait_time/1000000:.0f}"
 
+    if time_stamp_entry == 475822989:
+        a=1
     wait_time_exit = "0"
     parent = get_parent_if_last_for_level( context['parsedLines'],parsed_line_index)
     if parent != None:
@@ -753,9 +798,10 @@ def print_parsed_line(pc,d,parsed_line_index):
     time_ms = f'{int(time_ms):,}'
 
     if 'is_loop' in d:
-        ls = f"  for:  x{d['loop_links']}  " if d['loop_position'] == 1 else f"             "
-        output_string = output_string + utils.CYELLOW + f"{ls}{d['loop_position']}" + utils.CEND
-
+        ls = f"    x{d['loop_links']}-{d['loop_position']}"# if d['loop_position'] == 1 else f"             "
+        output_string = f"{output_string}   {utils.CYELLOW}{ls}{utils.CEND}"
+      #  ls = f"  for:  x{d['loop_links']}  " if d['loop_position'] == 1 else f"             "
+      #  output_string = output_string + utils.CYELLOW + f"{ls}{d['loop_position']}" + utils.CEND
     if element_time == "0":  element_time =''
     if soql_total_queries_delta ==0: soql_total_queries_delta = ''
     if soql_total_queries_exit ==0: soql_total_queries_exit = ''
@@ -765,7 +811,7 @@ def print_parsed_line(pc,d,parsed_line_index):
     if soql_total_queries_entry == 0: soql_total_queries_entry = ''
     if wait_time == "0":  wait_time =''
     if wait_time_exit == "0":  wait_time_exit ='' 
-    else: wait_time_exit=f"-{wait_time_exit}"
+    else: wait_time_exit=f"{wait_time_exit}"
     if soql_queries_cmt_entry == 0 : soql_queries_cmt_entry = ''
     if soql_queries_entry == 0 : soql_queries_entry = ''
 
@@ -773,8 +819,13 @@ def print_parsed_line(pc,d,parsed_line_index):
     time_stamp_exit = utils.CLIGHT_GRAY +utils.CFAINT+ f"{time_stamp_exit:12}" + utils.CEND
 
     s = utils.CLIGHT_GRAY +utils.CFAINT+  "|" + utils.CEND
+
+    times_lenght = 12
+
     if context['firstLineOut'] == True:
-        valname = f"{'wait':>5}     time query  Call Stack"
+       # valname = f"{'wait':>5}     time query  Call Stack"
+        valname = f"time         query  Call Stack"
+
         print(f"{utils.CFAINT}{'time entry':^12}{s}{utils.CFAINT}{'time exit':^12}{s}{'ts':^8}{s}{'CPU':^6}{s}{'cpuD':^6}{s}{'Qt':^3}{s}{'Q':^3}{s}{'Qmp':^3}{s}{'Qe':^3}{s}{'type':^21}{s}{'line':^4}{s}{valname:50}")
         context['firstLineOut'] = False
 
@@ -783,7 +834,16 @@ def print_parsed_line(pc,d,parsed_line_index):
     else:
         et_color = utils.CWHITE + utils.CFAINT
 
-    output_string = utils.CYELLOW+utils.CFAINT+ f"{wait_time:>5}{wait_time_exit:<4}{identation}"+et_color+f"{element_time:>5}"+utils.CDARK_GRAY+f"{soql_total_queries_delta:>4}" +utils.CEND + Cinit +f"    {output_string}"
+    a=f"{wait_time}·{element_time}·{wait_time_exit}"
+    lena = len(a)
+    aa = times_lenght-lena
+    if aa<0: aa=0
+  #  element_time = utils.CGREEN + utils.CFAINT+f"{wait_time}·{utils.CWHITE}{element_time}{utils.CGREEN}{utils.CFAINT}·{wait_time_exit:<{aa}}"
+    element_time = utils.CGREEN + f"{wait_time}·{utils.CWHITE}{element_time}{utils.CLIGHT_GREEN}·{wait_time_exit}{'':<{aa}}"
+
+   # output_string = utils.CYELLOW+utils.CFAINT+ f"{wait_time:>5}{wait_time_exit:<4}{identation}"+et_color+f"{element_time:>5}"+utils.CDARK_GRAY+f"{soql_total_queries_delta:>4}" +utils.CEND + Cinit +f"    {output_string}"
+   # output_string = utils.CYELLOW+utils.CFAINT+ f"{identation}"+f"{element_time:<{times_lenght}}"+utils.CYELLOW+utils.CFAINT+f"{soql_total_queries_delta:>4}" +utils.CEND + Cinit +f"    {output_string}"
+    output_string = utils.CYELLOW+utils.CFAINT+ f"{'':<{level}}"+f"{element_time:<{times_lenght}}"+utils.CYELLOW+utils.CFAINT+f"{soql_total_queries_delta:>4}" +utils.CEND + Cinit +f"    {utils.CYELLOW}{utils.CFAINT}{level} {utils.CEND}{Cinit}{output_string}"
 
     print(f"{time_stamp_entry:12}{s}{time_stamp_exit:12}{s}{time_ms:>8}{s}{cpu_time_entry:>6}{s}{cpu_time_elapsed:6}{s}{soql_total_queries_entry:>3}{s}{utils.CGREEN}{soql_queries_entry:>3}{utils.CEND}{s}{soql_queries_cmt_entry:>3}{s}{utils.CYELLOW}{(Qmct_estimate):>3}{utils.CEND}{s}{element_type_color}{element_type:21}{utils.CEND}{s}{apex_line_number:>4}{s}{output_string:50}"+utils.CEND)
 
