@@ -35,7 +35,8 @@ def checkBasketError():
       utils.raiseException(lc['response']['errorCode'],f"{inspect.stack()[1].function}: success:{lc['response']['success']}  isBasketValid:{lc['response']['isBasketValid']}")
   if 'result' in lc['response'] and 'offerDetails' in lc['response']['result']:
       utils.raiseException(f"CreateBasket {lc['response']['result']['offerDetails']['StatusCode']}",lc['response']['result']['offerDetails']['messages'][0])
-
+  if 'STATUS' in lc['response'] and lc['response']['STATUS'] == 'ERROR':
+     utils.raiseException(lc['response']['errorCode'],lc['response']['STATUS'])
   #if lc['response']['result']['error'] !='OK':
   #  print()
 
@@ -49,8 +50,6 @@ def checkOfferError():
   if 'success' in lc['response'] and lc['response']['success'] == False:
       utils.raiseException(lc['response']['errorCode'],f"{inspect.stack()[1].function}: success:{lc['response']['success']}  isBasketValid:{lc['response']['isBasketValid']}")
 
-
-
 #------------------------------------------------------------------------
 
 def getResult(obj):
@@ -93,10 +92,10 @@ def getOfferByCatalogue(catalogcode,contains=None,contextkey=None,context=None,f
 
 #------------------------------------------------------------------------
 
-def getOfferDetails(catalogcode,offercode,contextkey=None,context=None,forceinvalidatecache=None,isloggedin=None):
+def getOfferDetails(catalogcode,offercode,contextkey=None,context=None,forceinvalidatecache=None,isloggedin=None,ts_name=None):
   paramStr = stringify(locals(),exclude=['catalogcode','offercode'])
   action = f'/services/apexrest/{restClient.getNamespace()}/v3/catalogs/{catalogcode}/offers/{offercode}{paramStr}'
-  call = restClient.callAPI(action)
+  call = restClient.callAPI(action,ts_name=ts_name)
   checkOfferError()
 
 
@@ -114,18 +113,27 @@ def getOfferDetails(catalogcode,offercode,contextkey=None,context=None,forceinva
   lc['log'] = log
   return call
 #------------------------------------------------------------------------
-
-def createBasket(catalogcode,offer,contextKey=None,context=None,forceinvalidatecache=None,isloggedin=None,includeAttachment=None,returnBasket=None,validatebasket=None):
-  paramStr = stringify(locals(),exclude=['catalogcode','offer'])
+   
+def createBasket(catalogcode,offer,basketAction='AddWithNoConfig',productConfig=None,contextKey=None,context=None,forceinvalidatecache=None,isloggedin=None,includeAttachment=None,returnBasket=None,validatebasket=None,requestDateTime=None,rootAssetIds=None,canCreateBasket=False,ts_name=None):
+  paramStr = stringify(locals(),exclude=['catalogcode','offer','productConfig','requestDateTime','rootAssetIds','canCreateBasket'])
   action = f'/services/apexrest/{restClient.getNamespace()}/v3/catalogs/{catalogcode}/basket{paramStr}'
+
+  if basketAction not in ['AddWithNoConfig','AddAfterConfig','AssetToBasket']:
+     utils.raiseException('Invalid_Option',f'bakset Action {basketAction} is not a valid option.')
+
   restClient.glog().debug(action)
 
   body = {
-    "basketAction": "AddWithNoConfig",
-    "offer":offer
+    "basketAction": basketAction,
   }
+  if basketAction == 'AddWithNoConfig' :    body["offer"]=offer
+  if basketAction == 'AddAfterConfig' :    body["productConfig"]=productConfig
+  if basketAction == 'AssetToBasket' :    
+        body["rootAssetIds"]=rootAssetIds
+        body["requestDateTime"]=requestDateTime
 
-  response = restClient.callAPI(action, method="post", data=body)
+  response = restClient.callAPI(action, method="post", data=body,ts_name=ts_name)
+  filename = restClient.callSave('popopollll',logRequest=True,logReply=False)
   checkBasketError()
 
   lc = restClient.lastCall()
@@ -145,7 +153,19 @@ def createBasket(catalogcode,offer,contextKey=None,context=None,forceinvalidatec
 
 #------------------------------------------------------------------------
 
-def createBasketAfterConfig(catalogcode,offerDetails,contextKey=None,context=None,forceinvalidatecache=None,isloggedin=None,includeAttachment=None,returnBasket=None,validatebasket=None):
+def createBasketAfterConfig(catalogcode,offerDetails,contextKey=None,context=None,forceinvalidatecache=None,isloggedin=None,includeAttachment=None,returnBasket=None,validatebasket=None,ts_name=None):
+
+  res = createBasket(catalogcode,'',
+                    'AddAfterConfig',
+                    productConfig=getResult(offerDetails),
+                    context=contextKey,
+                    forceinvalidatecache=forceinvalidatecache,
+                    isloggedin=isloggedin,
+                    includeAttachment=includeAttachment,
+                    returnBasket=returnBasket,
+                    ts_name=ts_name)
+  
+  return res
   paramStr = stringify(locals(),exclude=['catalogcode','offerDetails'])
   action = f'/services/apexrest/{restClient.getNamespace()}/v3/catalogs/{catalogcode}/basket{paramStr}'
   restClient.glog().debug(action)
@@ -176,8 +196,18 @@ def createBasketAfterConfig(catalogcode,offerDetails,contextKey=None,context=Non
   return response
 
 #------------------------------------------------------------------------
+def createBasket_AssetToBasket():
+   print()
 
-def executeActions(basketAction):
+def getBasketDetails(catalogcode,basketkey,ts_name=None):
+  action = f'/services/apexrest/{restClient.getNamespace()}/v3/catalogs/{catalogcode}/basket/{basketkey}'
+  restClient.glog().debug(action)
+  response = restClient.callAPI(action, method="get",ts_name=ts_name)
+  checkBasketError()
+
+  return response
+   
+def executeActions(basketAction,ts_name=None):
   action = basketAction['link']
   body = basketAction['params']
   method="post"
@@ -185,15 +215,15 @@ def executeActions(basketAction):
   if basketAction['method'] == 'updateBasketAction':
     method = 'put'
 
-  response = restClient.callAPI(action, method=method, data=body)
-  response = checkBasketError(response)
+  response = restClient.callAPI(action, method=method, data=body,ts_name=ts_name)
+  checkBasketError()
   return response
 
 #------------------------------------------------------------------------
 
-def addChildBasket(basket,path):
+def addChildBasket(basket,path,ts_name=None):
   basketAction = digitalCommerceUtil.getAction(basket,path,method='addChildBasketAction')
-  basket2 = executeActions(basketAction)
+  basket2 = executeActions(basketAction,ts_name=ts_name)
 
   return basket2
 
@@ -211,8 +241,6 @@ def setError(error,errorCode):
     restClient.glog().error(f"Error {error}  {errorCode}")
     restClient.setLastCallError( error,errorCode)
 
-
-
 #------------------------------------------------------------------------
 
 def updateBasketQuantity(basket,path,value):
@@ -235,7 +263,6 @@ def updateBasketFields(basket,path,fieldsToBeUpdated):
 
   return basket2
 
-
 #  updates = [
 #    {
 #      "Category":'ACAT_EQUIPMENT',
@@ -264,7 +291,7 @@ def updateBasketAttributes(basket,path,updates):
 
 #------------------------------------------------------------------------
 
-def createCart(accountId,catalogCode,cartContextKey,createAsset=False,assetReferenceKey=None):
+def createCart(accountId,catalogCode,cartContextKey,createAsset=False,assetReferenceKey=None,ts_name=None):
   if type(cartContextKey) is dict:
     cartContextKey = cartContextKey
 
@@ -285,7 +312,7 @@ def createCart(accountId,catalogCode,cartContextKey,createAsset=False,assetRefer
       action = f'{action}&createAsset=true'
 
   #print(data)
-  call = restClient.callAPI(action, method="post", data=data)
+  call = restClient.callAPI(action, method="post", data=data,ts_name=ts_name)
   checkBasketError()
   return call
 
