@@ -41,6 +41,8 @@ def getAction(basket,path,method='addChildBasketAction'):
 
 
     obj = _getItemAtPath(basket,xpath)
+    if obj == '':
+        utils.raiseException('GET_ACTION_ERROR',f"PATH not found {xpath}. In path {path}")
     action = _getItemAtPath(obj,method,field='method')
 
     action['params']['offer'] = offer
@@ -79,13 +81,13 @@ def checkOffers(path=None,quantity=None,catcatalogueCode=None,account=None,baske
                 "__color__":utils.CEND
             }
             offers = digitalCommerce.getOfferByCatalogue(catalog['vlocity_cmt__CatalogCode__c'],pagesize=400)
-            times['elapsed'] = restClient.getLastCallTime()
+            times['elapsed'] = restClient.getLastCallElapsedTime()
             l = len(offers) if offers is not None else 0
             numOffersList.append(l)
             times['# Offers'] = l
 
         except Exception as e:
-            times['elapsed'] = restClient.getLastCallTime()
+            times['elapsed'] = restClient.getLastCallElapsedTime()
             times['Error'] = e.args[0]['error']
             times['__color__'] = utils.CRED
             getOfferTimes.append(times)
@@ -124,7 +126,7 @@ def checkOffers(path=None,quantity=None,catcatalogueCode=None,account=None,baske
         catsOffers.append(catOffers)
         getOfferTimes.append(times)
 
-    thread.processList(getOffersPerCatalog,catalogs,10)
+    thread.processList(getOffersPerCatalog,catalogs,1)
     utils.printFormated(getOfferTimes)
     totalOffers = sum(numOffersList)
 
@@ -231,11 +233,11 @@ def dba(offerCodes):
         }
 
         details = digitalCommerce.getOfferDetails(catalogCode,offerCode)
-        times['details'] = restClient.getLastCallTime()
+        times['details'] = restClient.getLastCallElapsedTime()
 
         if offerCodes['basketOps'] == True:
             basket = digitalCommerce.createBasket(catalogCode,offerCode)
-            times['createBasket'] = restClient.getLastCallTime()
+            times['createBasket'] = restClient.getLastCallElapsedTime()
 
             if details['result']['offerDetails']['offer']['offerType'] == 'Promotion':
                 try:
@@ -249,15 +251,15 @@ def dba(offerCodes):
                 offerDetails = updateOfferField(details,'0001','Quantity',4,'lineNumber')
 
             basket2 = digitalCommerce.createBasketAfterConfig(catalogCode,offerDetails)
-            times['afterConfig'] = restClient.getLastCallTime()
+            times['afterConfig'] = restClient.getLastCallElapsedTime()
         if 'accountId' in offerCodes and 1==2:
             times['cart']=''
             cart = digitalCommerce.createCart(offerCodes['accountId'],catalogCode,basket2['cartContextKey'],createAsset=False)
-            times['cart']=restClient.getLastCallTime()
+            times['cart']=restClient.getLastCallElapsedTime()
 
             times['delete']=''
             delete = CPQ.deleteCart(cart['orderId'])
-            times['delete']=restClient.getLastCallTime()
+            times['delete']=restClient.getLastCallElapsedTime()
 
             print()
 
