@@ -66,7 +66,9 @@ def getFromDebugList(context,values):
 def popFromList(context,type,value,key='key',endsWith=False):
     openItemsList = context['openItemsList']
     try:
-        for i,obj in enumerate(openItemsList):
+        #for i,obj in enumerate(openItemsList):
+
+        for i,obj in reversed(list(enumerate(openItemsList))):
             if obj['type'] == type:
                 if endsWith == True:
                     if key not in obj:      continue
@@ -279,11 +281,14 @@ def parseLimits(context):
         while '|' not in nextline:
             if 'SOQL queries' in nextline:
                 nlchunks = nextline.split(' ')
+                if f'{limitsNS}SOQL queries' not in context:
+                    context[f'{limitsNS}SOQL queries'] = 0
                 if int(context[f'{limitsNS}SOQL queries']) < int(nlchunks[6]):
                     context[f'{limitsNS}SOQL queries'] = nlchunks[6]
             if 'CPU time' in nextline:
                 nlchunks = nextline.split(' ')
-
+                if f'{limitsNS}CPU time' not in context:
+                    context[f'{limitsNS}CPU time'] = 0
                 if int(context[f'{limitsNS}CPU time']) < int(nlchunks[5]):
                     context[f'{limitsNS}CPU time'] = nlchunks[5]
             next = next + 1
@@ -475,6 +480,7 @@ def parseConstructor(context):
         return True
 
     if is_in_operation(context,'CONSTRUCTOR_EXIT'):
+
         decreaseIdent_pop_setFields(context,type='CONSTRUCTOR',key='output',value=chunks[5])
         return True
 
