@@ -1,6 +1,6 @@
 import os,sys,sys,gc
 from xmlrpc.client import Boolean
-from . import Sobjects, jsonFile,file, objectUtil,utils
+from . import Sobjects, jsonFile,file, objectUtil,utils,timeStats
 import requests,threading
 import logging,datetime,enum
 import simplejson as json
@@ -56,12 +56,12 @@ def _updateThreadCall(call):
     _calls[-1] = call
 
 def lastCall(field=None):
-    return thread_lastCall(field)
+    return _thread_lastCall(field)
 
 def thread_get_calls():
     return _get_thread_context()['calls']
 
-def thread_lastCall(field=None):
+def _thread_lastCall(field=None):
     """
     Returns the last rest call data (request, response, others).
     """
@@ -73,11 +73,11 @@ def thread_lastCall(field=None):
 def checklastThreadCallError(caller):
     """
     Raises and Exception if the last call has an error. """
-    lc = thread_lastCall()
+    lc = _thread_lastCall()
     if 'error' in lc and lc['error'] is not None:
         utils.raiseException(lc['errorCode'],lc['error'],caller)
 
-def get_call_times(call):
+def _get_call_times(call):
     et = call['elapsedTime']
     delta = call['deltaTime']
     times = {
@@ -85,23 +85,18 @@ def get_call_times(call):
         'delta':(delta.microseconds + delta.seconds * 1000000 )/1000,
     }
     return times 
-def getLastCallTimes():
-    t = thread_lastCall()
-  #  et = t['elapsedTime']
-  #  delta = t['deltaTime']
-  #  times = {
-  #      'elapsed':(et.microseconds + et.seconds * 1000000 )/1000,
-  #      'delta':(delta.microseconds + delta.seconds * 1000000 )/1000,
-  #  }
-    return get_call_times(t)
 
-def getLastCallTime():
-    return thread_lastCall_time()
-
-def thread_lastCall_time():
-    t = thread_lastCall('elapsedTime')
+def _thread_lastCall_time():
+    t = _thread_lastCall('elapsedTime')
     return (t.microseconds + t.seconds * 1000000 )/1000
 
+def getLastCallAllTimes():
+    call = _thread_lastCall()
+    return _get_call_times(call)
+
+def getLastCallElapsedTime():
+    return _thread_lastCall_time()
+
 def getConfigOrgsNames():
     """
     Get all names for the org in the config file."""
@@ -126,16 +121,18 @@ def loadConfigData():
     if incli is not None:
         setConfigFile(incli)
         return _configData
+    
+    root_folder = "incli"
 
-    _configDataName = os.path.abspath(".incli/IncliConf.json")
+    _configDataName = os.path.abspath(f"{root_folder}/IncliConf.json")
 
     if file.exists(_configDataName) == False:
         configData = {
             "folders": {
-                "input":os.path.abspath(".incli/input"),
-                "debug":os.path.abspath(".incli/debug"),
-                "output":os.path.abspath(".incli/output"),
-                "log":os.path.abspath(".incli/logs")
+                "input":os.path.abspath(f"{root_folder}/input"),
+                "debug":os.path.abspath(f"{root_folder}/debug"),
+                "output":os.path.abspath(f"{root_folder}/output"),
+                "log":os.path.abspath(f"{root_folder}/logs")
             },
             "orgs": []
         }
@@ -245,12 +242,19 @@ def init(userName_or_orgAlias,connectionName=None):
         try:
             glog().debug(f"Calling sfdx")
             success,obj,outputs = utils.execute_force_org_display(userName_or_orgAlias)
+            if success == False:
+                utils.raiseException('SFDXError',f"Connection failed. {outputs.stderr}")
+
+            if obj['Connected Status'] != 'Connected':
+                utils.raiseException('SFDXError',f"Connection failed. Connected Status:{obj['Connected Status']}")
+
             glog().debug(f"post sfdx")
 
+        except utils.InCliError as e:
+                raise e
         except Exception as e:
-            if e.strerror == 'No such file or directory':
-                utils.raiseException('SFDXError',"SFDX is not installed or it is not accesible.",other='https://developer.salesforce.com/docs/atlas.en-us.sfdx_setup.meta/sfdx_setup/sfdx_setup_install_cli.htm')
-            else:
+                if e.strerror == 'No such file or directory':
+                    utils.raiseException('SFDXError',"SFDX is not installed or it is not accesible.",other='https://developer.salesforce.com/docs/atlas.en-us.sfdx_setup.meta/sfdx_setup/sfdx_setup_install_cli.htm')
                 utils.raiseException('SFDXError',e.strerror)
 
         if success is False:
@@ -388,7 +392,6 @@ def _gerFolder(name):
         os.makedirs(folder)
     return folder
 
-#################################################################################################################################
 def _authenticate(login,isSandbox):
     if 'isSandBox' == None:
         raise ValueError(f"Environment connection parameters missing isSandBox field.")
@@ -400,7 +403,7 @@ def _authenticate(login,isSandbox):
     _get_thread_context('oauth2')
     call = requestRaw(url=f"https://{server}.salesforce.com/services/oauth2/token",method='post', parameters= login,headers=headers)
     
-    lc = thread_lastCall()
+    lc = _thread_lastCall()
     if lc['error'] is not None:
         utils.raiseException(lc['errorCode'],lc['error'],other=lc['errorOther'])
 
@@ -412,8 +415,29 @@ def _authenticate(login,isSandbox):
     glog().info(f"Authenticated. Bearer token {call['access_token']}")
 
     return call["instance_url"],call["access_token"]
+#################################################################################################################################
 
-def requestRaw(url,action=None,method = 'get',parameters = {},data={},headers={},access_token=None):
+ts = timeStats.TimeStats()
+def new_time_record():
+    global ts
+    ts.new()
+
+def time_print():
+    global ts
+    ts.print()
+
+def checkError():
+    lc = lastCall()
+
+    if 'error' in lc and lc['error'] != None and len(lc['error'])>10:
+        if 'response' in lc:
+            if 'serverResponse:' in lc['response']:
+                s = '{"a":' + lc['response'].split('serverResponse:')[-1] + "}"
+                obj = json.loads(s)
+                utils.raiseException(obj['a'][0]['errorCode'],obj['a'][0]['message'])
+        utils.raiseException(lc['errorCode'],lc['error'])
+
+def requestRaw(url,action=None,method = 'get',parameters = {},data={},headers={},access_token=None,ts_name=None):
     """
     Basic request Call. 
     
@@ -493,6 +517,9 @@ def requestRaw(url,action=None,method = 'get',parameters = {},data={},headers={}
         call['elapsedTime'] = r.elapsed
         call['elapsedTimeCall'] = r.elapsed
 
+        if ts_name != None:
+            ts.time_inner(ts_name,r.elapsed)
+
         if r.status_code < 300 :
             if r.text == '':
                 call['response'] = ''
@@ -513,13 +540,15 @@ def requestRaw(url,action=None,method = 'get',parameters = {},data={},headers={}
         call['elapsedTimeCall'] = call['deltaTime']
         call['status_code'] = 600
         call['response'] = 'No Response'
+        if ts_name != None:
+            ts.time_inner(ts_name,call['error'])
 
     if _currentConnectionName != 'ConnectionLess':
         _updateThreadCall(call)
 
     return call['response']
 
-def requestWithConnection(action,  parameters = {}, method = 'get', data = {},headers = {}):
+def requestWithConnection(action,  parameters = {}, method = 'get', data = {},headers = {},ts_name=None):
     """
     Performs a request using the current connection as configured in the file. 
     """
@@ -534,10 +563,10 @@ def requestWithConnection(action,  parameters = {}, method = 'get', data = {},he
                         method = method , 
                         data = data ,
                         headers  =headers,
-                        access_token=connection['access_token'] if 'access_token' in connection else None
-                        )
+                        access_token=connection['access_token'] if 'access_token' in connection else None,
+                        ts_name=ts_name)
 
-def callAPI_multi(action,params={} , method = 'get', data = {},headers={}):
+def callAPI_multi(action,params={} , method = 'get', data = {},headers={},ts_name=None):
     done = False
 
    # parameters = params
@@ -545,15 +574,15 @@ def callAPI_multi(action,params={} , method = 'get', data = {},headers={}):
     totalCalls = 0
 
     while done==False:
-        call = requestWithConnection(action,parameters = params,method=method, data=data, headers=headers)
+        call = requestWithConnection(action,parameters = params,method=method, data=data, headers=headers,ts_name=ts_name)
 
-        totalElepsedTime = thread_lastCall('elapsedTime') + totalElepsedTime
+        totalElepsedTime = _thread_lastCall('elapsedTime') + totalElepsedTime
         totalCalls = totalCalls + 1
 
-        if  call == None or thread_lastCall('status_code')>=300:
+        if  call == None or _thread_lastCall('status_code')>=300:
             break
         
-        glog().debug(f"callAPI_multi: <{action}>  ts:{thread_lastCall('elapsedTime')}") 
+        glog().debug(f"callAPI_multi: <{action}>  ts:{_thread_lastCall('elapsedTime')}") 
 
         #For chainable integration procedures
         if type(call) is dict and 'IPResult' in call and 'vlcStatus' in call['IPResult'] and call['IPResult']['vlcStatus'] == 'InProgress':
@@ -569,7 +598,7 @@ def callAPI_multi(action,params={} , method = 'get', data = {},headers={}):
         else:
             done = True
 
-    lc = thread_lastCall()
+    lc = _thread_lastCall()
     lc['elapsedTime'] = totalElepsedTime
     lc['totalCalls'] = totalCalls    
     _updateThreadCall(lc)
@@ -587,8 +616,8 @@ def callAPI_multi(action,params={} , method = 'get', data = {},headers={}):
 #def callAPI_debug(action, parameters = {}, method = 'get', data = {}, headers={}):    
     return callAPI(action,parameters,method,data,headers)
 
-def callAPI(action, parameters = {}, method = 'get', data = {}, headers={}):    
-    call = callAPI_multi(action,parameters,method,data,headers)
+def callAPI(action, parameters = {}, method = 'get', data = {}, headers={},ts_name=None):    
+    call = callAPI_multi(action,parameters,method,data,headers,ts_name=ts_name)
     return call
 
 def callSave(logFileName,logRequest=False,logReply=True,timeStamp=False,responseProcessing=None,requestProcessing=None):
@@ -602,7 +631,7 @@ def callSave(logFileName,logRequest=False,logReply=True,timeStamp=False,response
     request_filename = None
     response_filename = None
 
-    lc = thread_lastCall()
+    lc = _thread_lastCall()
 
     if logRequest == True:
         fn = f'{filename}_req'
