@@ -11,11 +11,16 @@ class TimeStats:
         if fields == None: return
         for field in fields:
             self.time_record[field] = ''
-    def time(self,field):
-        self.time_record[field] = restClient.getLastCallTime()
     
+    def time(self,field):
+        self.time_record[field] = restClient.getLastCallElapsedTime()
+
+    def time_inner(self,field,time):
+        self.time_record[field] = time
+
     def time_no(self,field,value):
         self.time_record[field] = value
+    
     def print(self):
         utils.printFormated(self.time_records)
 
