@@ -59,7 +59,7 @@ def set_incli_traceFlag_for_user(userF):
     DebugLevelId = tooling.idF('DebugLevel',"DeveloperName:InCli")   
     if DebugLevelId == None:
         call = create_debug_level_incli()
-        DebugLevelId = call['Id']  
+        DebugLevelId = call['id']  
 
     q = f"select Id from TraceFlag where DebugLevelId='{DebugLevelId}' and TracedEntityId = '{userId}'"
     call = tooling.query(q)
