@@ -67,6 +67,9 @@ def executeCommand(command):
         list_files = subprocess.run(command,capture_output=True,text=True)
 
     print(f"{CFAINT}{list_files.stderr}{CEND}")
+    if "ERROR" in list_files.stderr:
+        error = list_files.stderr.split('ERROR')[1]
+        raiseException("SFDXError",f"ERROR{error}")
     return list_files
 
 def execute_force_org_display(userName_or_orgAlias):
@@ -182,7 +185,21 @@ def printException(exception):
         _str = simplejson.dumps(error, indent=2)
     logging.error(_str)
 
+def print_json(obj,filename=None):
+   # input_data = simplejson.loads(obj)
+    json_formatted_str = simplejson.dumps(obj, indent=2, ensure_ascii=False)
 
+    if filename != None:
+        original_stdout = sys.stdout
+        _filename = f"{filename}.json"
+        with open(_filename, 'w') as f:
+            sys.stdout = f 
+            print(json_formatted_str)
+            sys.stdout = original_stdout 
+            print()
+            print(f"   File {_filename} created.")
+    else:  
+        print(json_formatted_str)
 #----------------------------------------------------------------------------------------------------
 
 def throwOrLog(message,raiseEx=True,dumps=False):
@@ -240,7 +257,7 @@ def deleteNulls(obj,systemFields=True,nulls=True,delAttributes=True):
             res[key] = obj[key]
     return res
 
-def printFormated(records,fieldsString=None,rename=None,exclude=None,checkNumber=False,separator=':'):
+def printFormated(records,fieldsString=None,rename=None,exclude=None,checkNumber=False,separator=':',print_renames=True):
     """
     - fieldsString: format field%name:field2%name2:field3
         field is the field in the obj. Name is how it will be printed (the short version). 
@@ -270,7 +287,7 @@ def printFormated(records,fieldsString=None,rename=None,exclude=None,checkNumber
         for ren in renames:
             chunks = ren.split('%')
             _renames[chunks[0]] = chunks[1]
-            print( CFAINT +  f"Column {chunks[0]} as {chunks[1]}" +CEND)
+            if print_renames: print( CFAINT +  f"Column {chunks[0]} as {chunks[1]}" +CEND)
         print()
     
     _fields = {}
@@ -361,12 +378,16 @@ def today_date(format="%Y-%m-%d"):
     today = datetime.today()
     return today.strftime(format)  
 
-def future_date(years=None,days=None,format="%Y-%m-%d"):
+def datetimestr_around(str,minutes,format = '%Y-%m-%dT%H:%M:%S'):
+    dt = datetime.strptime(str,format)
+    dt_before = dt - timedelta(minutes=minutes) 
+    dt_after = dt + timedelta(minutes=minutes) 
+    return dt_before.strftime(format),dt_after.strftime(format)
+
+def future_date(years=None,days=None,minutes=None,format="%Y-%m-%d"):
     today = datetime.today()
-    if years!=None:
-        future = today.replace(year=today.year + years)
-    if days!=None:
-        future = today.replace(day=today.day + days)
+    if years!=None:  future = today.replace(year=today.year + years)
+    if days!=None:    future = today.replace(day=today.day + days)
     return future.strftime(format)  
 
 def callerPythonFileFolder():
@@ -376,7 +397,6 @@ def callerPythonFileFolder():
         caller_filefolder = dirname(caller_filefolder)
     return caller_filefolder
 
-
 def getEntryDirectory():
     return sys.path[0]
 
@@ -450,7 +470,6 @@ def Obj(var,onlyOne=True):
                 i = i + 1
             return _objs
 
-
 def computeTimes(call,name,times,_type=None):
     global _counter
 
@@ -482,8 +501,76 @@ def computeTimes(call,name,times,_type=None):
     print (sec)
     return debugTimes
 
+def _is_match(objs,x,y,field):
+    ox = objs[x]
+    oy = objs[y]
+
+    if 'ident' not in ox: return False
+    if 'ident' not in oy: return False
+
+    if ox['ident'] != oy['ident'] : return False
+
+    if ox[field] != oy[field]:  return False
+    return True
+def _is_next_link(objs,field,chain_start,link_lenght,posible_link_start):
+    for z in range(0,link_lenght):
+        if posible_link_start+z >= len(objs): return False
+        if _is_match(objs,chain_start+z,posible_link_start+z,field) == False: return False
+
+    return True
+
 def get_all_loops(objs,field):
 
+    all_loops = []
+    _all_loops = []
+
+    lastX = 0
+
+    for x,val in enumerate(objs):
+        if x < lastX:    continue
+        if objs[x]['type'] == 'SOQL':continue
+        loop_lenght = None
+        lastDelta = 0
+        loop = []
+
+        max = 30
+        is_loop = False
+        
+        ox = objs[x]
+        if 'timeStamp' in ox and ox['timeStamp'][0] == 10189843723:
+            a=1
+        for y in range(x+1,len(objs)):
+            if is_loop == False and y-x>=max:
+                break
+            oy = objs[y]
+            if 'ident' in ox and ox['ident'] > oy['ident']:
+                break        
+            #    print(f"{x}:{ox['output']} {y}:{oy['output']}")
+  
+            
+            if _is_match(objs,x,y,field):
+                chain_start = x
+                next_link_start = y
+                link_lenght = y-x
+                loop = []
+
+                while True:
+                    if _is_next_link(objs,field,chain_start,link_lenght,next_link_start):
+                        if len(loop) == 0: loop.append(x)
+                        loop.append(next_link_start)
+                        next_link_start = next_link_start + link_lenght
+                        lastX = next_link_start
+                    else:
+                        break
+                if len(loop) > 0:  
+                    all_loops.append(loop)
+                    break
+
+
+    return all_loops
+    print(all_loops)
+def get_all_loops1(objs,field):
+
     all_loops = []
     lastX = 0
   #  if objs==None:
@@ -498,7 +585,7 @@ def get_all_loops(objs,field):
    #         oo = objs[x]
    #     objx = objs[x]['output']
 
-        max = 8
+        max = 30
         is_loop = False
 
         for y in range(x+1,len(objs)):
@@ -510,9 +597,17 @@ def get_all_loops(objs,field):
 
      #       if field not in objs[y]:
      #           _o = objs[y]
-            if 'ident' in objs[y] and 'ident' in objs[x] and objs[y]['ident'] == objs[x]['ident'] and objs[y][field] == objs[x][field]:
+            ox = objs[x]
+            oy = objs[y]
+            if _is_match(objs,x,y,field):
+          #  if 'ident' in objs[y] and 'ident' in objs[x] and objs[y]['ident'] == objs[x]['ident'] and objs[y][field] == objs[x][field]:
       #          line = objs[y][field]
-
+                posible_link_start = x
+                posible_link_end = y
+                if y == 1445:
+                    a=1
+                if y == 1575:
+                    a=1
                 if loop_lenght == None:
                     loop_lenght = y-x
                     lastDelta = loop_lenght
@@ -530,12 +625,13 @@ def get_all_loops(objs,field):
                         loop.append(y)
                 else:  break
                 if is_loop == True:     lastX = y 
-
+         #   else:
+         #       if is_loop:
+         #           is_loop = False
+         #           break
         if len(loop) > 0:  all_loops.append(loop)
 
     return all_loops
-    print(all_loops)
-
 def datetime_now_string(format = '%Y-%m-%dT%H:%M:%S%z',addMinutes=0):
     tz2 = datetime.now().astimezone()
     if addMinutes != 0:
