@@ -38,14 +38,14 @@ def do_work(details):
         call1 = restClient.lastCall()
         if call1['totalCalls'] >1:
           calls = restClient.thread_get_calls()
-          times = restClient.get_call_times(call1)
-          times2 = restClient.get_call_times(calls[-2])
+          times = restClient._get_call_times(call1)
+          times2 = restClient._get_call_times(calls[-2])
           ts.time_no("Basket1",times2['delta'])
           ts.time_no("Basket2",times['delta'])
 
        #   print(f"{times2} {times}",flush=True)
 
-     #   print(restClient.getLastCallTimes(),flush=True)
+     #   print(restClient.getLastCallAllTimes(),flush=True)
       except Exception as e:
         utils.printException(e)
         errors = errors + 1
@@ -97,6 +97,7 @@ class Test_Query(unittest.TestCase):
 
       utils.printFormated(res['records'])
       print()
+    
     def test_createCart(self):
         restClient.init('NOSDEV')
 
@@ -183,7 +184,7 @@ class Test_Query(unittest.TestCase):
         sim_m['vlocity_cmt__BillingAccountId__c'] = '0013O000016sKfwQAE'
 
         res = CPQ.updateCartItem_api(cartId,items)
-        time = restClient.getLastCallTime()
+        time = restClient.getLastCallElapsedTime()
         print(time)
         print(res['messages'][0]['message'])
 
@@ -239,7 +240,7 @@ class Test_Query(unittest.TestCase):
 
         items = {"records":objs}
         res = CPQ.updateCartItem_api(cartId,items)
-        time = restClient.getLastCallTime()
+        time = restClient.getLastCallElapsedTime()
         restClient.callSave('minimum',logRequest=True)
         print(time)
         print(res['messages'][0]['message'])
@@ -314,7 +315,7 @@ class Test_Query(unittest.TestCase):
 
         items = {"records":[C_NOS_OFFER_001]}
         res = CPQ.updateCartItem_api(cartId,items)
-        print(restClient.getLastCallTime())
+        print(restClient.getLastCallElapsedTime())
         print(res['messages'][0]['message'])
 
         incli._main(["-u","NOSDEV","-q", f"select Id,vlocity_cmt__BillingAccountId__c,vlocity_cmt__LineNumber__c,vlocity_cmt__ServiceAccountId__c,vlocity_cmt__Product2Id__r.productCode from orderitem where OrderId='{cartId}' order by vlocity_cmt__LineNumber__c","-fields","all"])  
@@ -394,6 +395,7 @@ class Test_Query(unittest.TestCase):
               li = self.lineItem_for_update(record)
               obj['lineItems']['records'].append(li)
           return obj
+    
     def test_do_all_puts(self):
         def getObj(lineItem,account2Id):
           obj = {}
@@ -406,6 +408,8 @@ class Test_Query(unittest.TestCase):
           obj['vlocity_cmt__ServiceAccountId__c'] = account2Id
           obj['vlocity_cmt__BillingAccountId__c'] = account2Id
 
+          obj['attributeCategories'] = lineItem['attributeCategories']
+
           if 'lineItems2' in lineItem:
             obj['lineItems'] = lineItem['lineItems']
             for li in obj['lineItems']['records']:
@@ -473,10 +477,10 @@ class Test_Query(unittest.TestCase):
 
         accountname='unaiTest_1'
 
-        cartId = self.create_cart_with_promo(accountname=accountname,cartName='testa_11',promoName='NOS4u 100Mb + Móvel',create=True,pricelistF='Name:B2C Price List')
+        cartId = self.create_cart_with_promo(accountname=accountname,cartName='testa_18',promoName='NOS4u 100Mb + Móvel',create=True,pricelistF='Name:B2C Price List')
 
         all = CPQ.getCartItems_api(cartId,fields='vlocity_cmt__ServiceAccountId__c,vlocity_cmt__BillingAccountId__c',price=False,validate=False,includeAttachment=False)
-        print(restClient.getLastCallTime())
+        print(restClient.getLastCallElapsedTime())
 
         account1Id = account.getId(f'name:{accountname}')
         account2Id = account.getId('name:unai2')
@@ -488,7 +492,7 @@ class Test_Query(unittest.TestCase):
         print(cartId)
 
         codes = ['C_NOS_SERVICE_TV_003','C_NOS_EQUIP_TV_006','C_NOS_EQUIP_TV_009','C_NOS_SERVICE_IF_001','C_NOS_EQUIP_GIGA_ROUTER','C_NOS_DN','C_NOS_SERVICE_VF_001','C_NOS_MSISDN','C_NOS_SERVICE_VM_001','C_NOS_EQUIP_SIM_CARD','C_NOS_OFFER_001','C_NOS_OFFER_006','C_NOS_AGG_SERVICE_TV_UMA','C_NOS_AGG_SERVICE_IF_HFC','C_NOS_AGG_SERVICE_VF_HFC','C_NOS_AGG_SERVICES_MAND_VM']
-        codes = ['C_NOS_OFFER_006']
+    #    codes = ['C_NOS_OFFER_006']
       #  codes = []
         lineItem = objectUtil.getSiblingWhere(items,selectKey='ProductCode',selectKeyValue='C_NOS_OFFER_001',whereKey='itemType',whereValue='lineItem')['object']
    #     for key in lineItem.keys():
@@ -519,11 +523,14 @@ class Test_Query(unittest.TestCase):
         res = CPQ.updateCartItem_api(cartId,items)  
         filename = restClient.callSave('records0001',logRequest=True,logReply=False)   
 
-        print(restClient.getLastCallTime())
+        print(restClient.getLastCallElapsedTime())
         print(res['messages'][0]['message'])
 
         incli._main(["-u","NOSDEV","-q", f"select Id,vlocity_cmt__BillingAccountId__c,vlocity_cmt__LineNumber__c,vlocity_cmt__ServiceAccountId__c,vlocity_cmt__Product2Id__r.productCode from orderitem where OrderId='{cartId}' order by vlocity_cmt__LineNumber__c","-fields","all"])  
 
+        items = CPQ.getCartItems_api(cartId=cartId)
+        print()
+
     def test_get_order_lines_for_CartId(self):
         cartId = '8013O000003j97nQAA'
         incli._main(["-u","NOSDEV","-q", f"select Id,vlocity_cmt__LineNumber__c,vlocity_cmt__Product2Id__r.productCode,vlocity_cmt__Product2Id__r.name from orderitem where OrderId='{cartId}' order by vlocity_cmt__LineNumber__c","-fields","all"])  
@@ -539,7 +546,28 @@ class Test_Query(unittest.TestCase):
     def test_delete_discount(self):
 
       restClient.init('NOSDEV')
-      
+
+    def create_cart_with_offer(self,accountname,cartName,offerName,create=False,pricelistF='Name:B2C Price List'):
+
+        cartId = CPQ.getCartId(f"name:{cartName}")
+        if cartId != None:
+          if create == False:
+            return cartId
+          CPQ.deleteCart(cartId)
+
+        accountF = f'Name:{accountname}'
+        accountId = account.create_Id(accountname,recordTypeName='Consumer')
+
+        inputFields = {
+           'NOS_t_CoverageTechnology__c':'FTTH'
+        }
+        cartId = CPQ.createCart(accountF= accountId, pricelistF=pricelistF,name=cartName,checkExists=True,inputFields=inputFields)
+
+        offer = CPQ.getCartProducts(cartId,query=offerName,onlyOne=True)
+        res = CPQ.addItemstoCart(cartId,productCode=offer['ProductCode']['value'],pricelistF=pricelistF)
+
+        return cartId  
+    
     def create_cart_with_promo(self,accountname,cartName,promoName,create=False,pricelistF='Name:B2C Price List'):
 
         cartId = CPQ.getCartId(f"name:{cartName}")
@@ -552,7 +580,6 @@ class Test_Query(unittest.TestCase):
         accountId = account.create_Id(accountname,recordTypeName='Consumer')
         cartId = CPQ.createCart(accountF= accountId, pricelistF=pricelistF,name=cartName,checkExists=True)
 
-     #   promoName = 'NOS4u 100Mb + Móvel'
         promo = CPQ.getCartPromotions(cartId,query=promoName,onlyOne=True)
 
         res = CPQ.postCartsPromoItems_api(cartId,promo['Id'],price=False,validate=False)
@@ -560,33 +587,34 @@ class Test_Query(unittest.TestCase):
         return cartId
 
     def test_post_cart_item(self):
-        restClient.init('NOSQSM')
+        restClient.init('NOSDEV')
         restClient.setLoggingLevel()
-        cartName = 'testa_4'
+        cartName = 'testa_4c'
 
         cartId = CPQ.getCartId(f"name:{cartName}")
         if cartId != None:
           CPQ.deleteCart(cartId)
 
-        accountId = account.createAccount_Id('Name:unaiTest2',recordTypeName='Consumer')
+        accountId = account.create_Id('unaiTest1c',recordTypeName='Consumer')
         cartId = CPQ.createCart(accountF= accountId, pricelistF='Name:B2C Price List',name=cartName,checkExists=True)
-        print(restClient.getLastCallTime())
+        print(restClient.getLastCallElapsedTime())
 
-        promoName = 'NOS4u 100Mb + Móvel'
+        promoName = 'NOS4u 100Mb + Móvel - Desc. Mensal'
         promo = CPQ.getCartPromotions(cartId,query=promoName,onlyOne=True)
 
         res = CPQ.postCartsPromoItems_api(cartId,promo['Id'])
-        print(restClient.getLastCallTime())
+        print(restClient.getLastCallElapsedTime())
 
         all = CPQ.getCartItems_api(cartId,fields='vlocity_cmt__ServiceAccountId__c,vlocity_cmt__BillingAccountId__c',price=False,validate=False,includeAttachment=False)
-        print(restClient.getLastCallTime())
+        print(restClient.getLastCallElapsedTime())
 
     def test_create_cart_from_asset(self):
-        restClient.init('NOSQSM')
+        restClient.init('DEVNOSCAT2')
+
         restClient.setLoggingLevel()
         cartName = 'testa_4'
 
-        cartId = CPQ.createCartFromAsset(assetId='02i7a00000SQHHNAA5',accountId='0017a000029BaSPAA0',date='2023-01-20 23:00:00',inputFields={"Name":cartName})
+        cartId = CPQ.createCartFromAsset(assetId='02i3O00000KQ489QAD',accountId='0013O0000186LXLQA2',date='2023-02-22',inputFields={"Name":cartName})
 
         print()
 
@@ -624,55 +652,55 @@ class Test_Query(unittest.TestCase):
       #  line2['vlocity_cmt__Action__c'].pop('label')
        # line2['vlocity_cmt__Action__c'].pop('fieldName')  #REQ
 
+        if 1==1:
+          line2['attributeCategories'] = lineItem['attributeCategories']
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][2]['userValues'] = 'yyyy5'
+          line2['attributeCategories']['records'].pop()
+          line2['attributeCategories']['records'].pop()
+          line2['attributeCategories']['records'][0]['productAttributes']['records'].pop(0)
+          line2['attributeCategories']['records'][0]['productAttributes']['records'].pop(0)
 
-        line2['attributeCategories'] = lineItem['attributeCategories']
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][2]['userValues'] = 'yyyy5'
-        line2['attributeCategories']['records'].pop()
-        line2['attributeCategories']['records'].pop()
-        line2['attributeCategories']['records'][0]['productAttributes']['records'].pop(0)
-        line2['attributeCategories']['records'][0]['productAttributes']['records'].pop(0)
+          line2['attributeCategories']['records'][0]['productAttributes'].pop('totalSize')
+          line2['attributeCategories']['records'][0]['productAttributes'].pop('messages')
 
-        line2['attributeCategories']['records'][0]['productAttributes'].pop('totalSize')
-        line2['attributeCategories']['records'][0]['productAttributes'].pop('messages')
+          line2['attributeCategories']['records'][0].pop('displaySequence')
+          line2['attributeCategories']['records'][0].pop('Name')
+          line2['attributeCategories']['records'][0].pop('id')
+          line2['attributeCategories']['records'][0].pop('messages')
 
-        line2['attributeCategories']['records'][0].pop('displaySequence')
-        line2['attributeCategories']['records'][0].pop('Name')
-        line2['attributeCategories']['records'][0].pop('id')
-        line2['attributeCategories']['records'][0].pop('messages')
+        # line2['attributeCategories']['records'][0].pop('Code__c')  REQ
 
-       # line2['attributeCategories']['records'][0].pop('Code__c')  REQ
-
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('messages')
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('dataType')
-     #   line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('inputType',None)   REQ
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('hasRules')
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('cloneable')
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('multiselect')
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('isNotTranslatable')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('messages')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('dataType')
+      #   line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('inputType',None)   REQ
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('hasRules')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('cloneable')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('multiselect')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('isNotTranslatable')
 
 
-     #   line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('required')     REQ
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('attributeId')
+      #   line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('required')     REQ
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('attributeId')
 
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('filterable')
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('label')
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('displaySequence')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('filterable')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('label')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('displaySequence')
 
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('disabled',None)
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('readonly',None)
-        line2['attributeCategories'].pop('totalSize')
-        line2['attributeCategories'].pop('messages')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('disabled',None)
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('readonly',None)
+          line2['attributeCategories'].pop('totalSize')
+          line2['attributeCategories'].pop('messages')
 
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('values')
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('hidden',None)
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('disabled',None)
-        line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('readonly',None)
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('values')
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('hidden',None)
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('disabled',None)
+          line2['attributeCategories']['records'][0]['productAttributes']['records'][0].pop('readonly',None)
 
         items = {"records":[line2]}
         res = CPQ.updateCartItem_api(cartId,items)  
         filepath = restClient.callSave("update1111",logRequest=True,logReply=False)
         print(res['messages'][0]['message'])
-        print(restClient.getLastCallTime())
+        print(restClient.getLastCallElapsedTime())
 
         itemsAfter = CPQ.getCartItems_api(cartId,price=False,validate=False)
 
@@ -716,16 +744,29 @@ class Test_Query(unittest.TestCase):
         print()
 
     def test_get_cart_items(seld):
-        restClient.init('NOSQSM')
+        restClient.init('DEVNOSCAT4')
         restClient.setLoggingLevel()
-        cartId = "8013O000003izGtQAI"
+        cartId = "8013O000003vgS1QAI"
 
-        items = CPQ.getCartItems_api(cartId,price=True,validate=True)
+        items = CPQ.getCartItems_api(cartId,price=False,validate=False)
+
+        filename = restClient.callSave('getItems')
 
         print()
 
+    def test_get_cart_items_bt(seld):
+        restClient.init('DEVNOSCAT4')
+        restClient.setLoggingLevel()
+        cartId = "8013O000003vgS1QAI"
+
+        items = CPQ.getCartItems_api_bt(cartId,price=True,validate=True)
+        filename = restClient.callSave('getItems_bt')
+
+        print()
+
+
     def test_submit_order(seld):
-        restClient.init('NOSDEV')
+        restClient.init('DEVNOSCAT2')
         restClient.setLoggingLevel()
         cartId = "8013O000003izGtQAI"
 
@@ -733,16 +774,27 @@ class Test_Query(unittest.TestCase):
 
         print()
 
+    def test_post_cart_items_Movel(self):
+        restClient.init('NOSDEV')
+        pricelistF = "Name:Woo Price List"
+
+        self.create_cart_with_offer('account_2','cart_a4','Móvel',pricelistF=pricelistF,create=False)
+
+        print()
+
+
     def test_post_cart_items(self):
-        restClient.init('DTI')
+        restClient.init('DEVNOSCAT4')
         restClient.setLoggingLevel()
         pricelistF = "Name:B2C Price List"
 
         ts = timeStats.TimeStats()
 
-        cartId = self.create_cart_with_promo('account_2','cart_3','NOS4u 100Mb + Móvel',pricelistF=pricelistF,create=True)
+        cartId = self.create_cart_with_promo('account_2','cart_7a2','NOS4u 100Mb + Móvel',pricelistF=pricelistF,create=True)
         print(cartId)
-        items = CPQ.getCartItems_api(cartId,price=False,validate=False)
+     #   items = CPQ.getCartItems_api(cartId,price=False,validate=False)
+        items = CPQ.getCartItems_api_bt(cartId,price=False,validate=False)
+
         req1,res1 = restClient.callSave('getITems123')
         parentRecord = items['records'][0]['productGroups']['records'][0]
 
@@ -774,4 +826,80 @@ class Test_Query(unittest.TestCase):
           ts.time('deleteCartItem')
 
         ts.print()
-        print()
\ No newline at end of file
+        print()
+
+    def test_getCartItems_x(self):
+        restClient.init('NOSDEV')
+        q="""SELECT id,
+                    pricebook2id,
+                    accountid,
+                    createddate,
+                    vlocity_cmt__defaultcurrencypaymentmode__c,
+                    vlocity_cmt__effectiverecurringtotal__c,
+                    vlocity_cmt__effectiveonetimetotal__c,
+                    vlocity_cmt__numberofcontractedmonths__c,
+                    vlocity_cmt__pricelistid__c,
+                    recordtypeid,
+                    recordtype.developername,
+                    status,
+                    effectivedate,
+                    vlocity_cmt__requestdate__c,
+                    vlocity_cmt__orderstatus__c,
+                    vlocity_cmt__ischangesallowed__c,
+                    vlocity_cmt__ischangesaccepted__c,
+                    vlocity_cmt__supplementalaction__c,
+                    vlocity_cmt__supersededorderid__c,
+                    vlocity_cmt__firstversionorderidentifier__c,
+                    vlocity_cmt__requestedstartdate__c,
+                    vlocity_cmt__originatingcontractid__c,
+                    vlocity_cmt__lastpricedat__c,
+                    vlocity_cmt__validationdate__c,
+                    vlocity_cmt__ordergroupid__c,
+                    vlocity_cmt__ordergroupid__r.vlocity_cmt__groupcartid__c,
+                    nos_t_scenariolevel__c,
+                    nos_t_coveragetechnology__c,
+                    nos_t_businessscenario__c,
+                    nos_t_channel__c,
+                    nos_t_process__c,
+                    nos_b_isbsimulation__c 
+                    FROM Order WHERE Id IN ('8013O000003keqvQAA')"""
+        res = query.query(q)
+
+        print(restClient.getLastCallAllTimes())
+
+        q2 =f"""SELECT Id,
+                      Quantity,
+                      vlocity_cmt__AssetReferenceId__c,
+                      vlocity_cmt__ParentItemId__c,
+                      vlocity_cmt__RootItemId__c,
+                      PriceBookEntry.Product2Id,
+                      PriceBookEntry.Name,
+                      PriceBookEntry.Product2.vlocity_cmt__GlobalGroupKey__c,
+                      vlocity_cmt__LineNumber__c,
+                      vlocity_cmt__Product2Id__c,
+                      vlocity_cmt__Action__c,
+                      PricebookEntryId,
+                      vlocity_cmt__ProvisioningStatus__c,
+                      vlocity_cmt__ProductHierarchyPath__c,
+                      vlocity_cmt__ProductHierarchyGroupKeyPath__c,
+                      vlocity_cmt__CatalogItemReferenceDateTime__c,
+                      OrderId,
+                      vlocity_cmt__SupplementalAction__c,
+                      vlocity_cmt__IsChangesAllowed__c 
+                      FROM OrderItem WHERE OrderId = '{res['records'][0]['Id']}'  ORDER BY vlocity_cmt__LineNumber__c """
+        res2 = query.query(q2)
+        print(restClient.getLastCallAllTimes())
+
+        idl = [r['Id'] for r in res2['records']]
+        lineItemIds = query.IN_clause(idl)
+
+        q3 = f"""SELECT vlocity_cmt__OrderAppliedPromotionId__r.vlocity_cmt__PromotionId__c,
+                        vlocity_cmt__OrderAppliedPromotionId__r.vlocity_cmt__Action__c,
+                        vlocity_cmt__OrderItemId__c 
+                        FROM vlocity_cmt__OrderAppliedPromotionItem__c 
+                        WHERE vlocity_cmt__OrderItemId__c IN ({lineItemIds})"""
+        res3 = query.query(q3)
+
+        print(restClient.getLastCallAllTimes())
+        print()
+
