@@ -76,12 +76,18 @@ class Test_Main(unittest.TestCase):
 
     def test_o_name(self):
         try:
-            InCli._main(["-u","uormaechea.devnoscat2@nos.pt","-o","-name","order"])
+            InCli._main(["-u","uormaechea.devnoscat4@nos.pt","-o","-name","order"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+    def test_o_name_limit(self):
+        try:
+            InCli._main(["-u","uormaechea.devnoscat4@nos.pt","-o","-name","order","-limit","10"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
             self.assertTrue(1==2)
-
     def test_o_like(self):
         try:
             InCli._main(["-u","uormaechea.devnoscat2@nos.pt","-o","-like","XOM"])
@@ -108,7 +114,7 @@ class Test_Main(unittest.TestCase):
 
     def test_cc(self):
         try:
-            InCli._main(["-u","NOSDEV","-cc"])
+            InCli._main(["-u","DEVNOSCAT4","-cc"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -163,6 +169,14 @@ class Test_Main(unittest.TestCase):
             print(traceback.format_exc())
             self.assertTrue(1==2)
 
+    def test_logs_where(self):
+        try:
+            InCli._main(["-u","NOSDEV","-logs","-where","adaasdasd='asasd'"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
     def test_logs_store(self):
         try:
             InCli._main(["-logs","-store"])
@@ -173,7 +187,7 @@ class Test_Main(unittest.TestCase):
 
     def test_logs_store_id(self):
         try:
-            InCli._main(["-logs","-store","07L3O00000Dwi37UAB"])
+            InCli._main(["-logs","-store","07L0Q00000N7AHsUAN"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -293,7 +307,14 @@ class Test_Main(unittest.TestCase):
 #07L3O00000Dy7SAUAZ
     def test_log_ID(self):
         try:
-            InCli._main(["-u","NOSDEV","-logs","07L3O00000DyP4iUAF"])
+            InCli._main(["-u","DEVNOSCAT4","-logs","07L3O00000E7b0YUAR"])
+        except Exception as e:
+            print(e)
+            print(traceback.format_exc())
+
+    def test_log_ID_wrong(self):
+        try:
+            InCli._main(["-u","DEVNOSCAT4","-logs","xxx"])
         except Exception as e:
             print(e)
             print(traceback.format_exc())
@@ -330,12 +351,20 @@ class Test_Main(unittest.TestCase):
     
     def test_log_file(self):
         try:
-            id = '07L0Q00000N5yVKUAZ'
-            res = InCli._main(["-logs","-inputfile",f"/Users/uormaechea/Downloads/logs_SF/apex-07L7T000003AWSiUAO.log"])
+            res = InCli._main(["-logs","-inputfile",f"/Users/uormaechea/Documents/Dev/python/InCliLib/InCLipkg/incli/logs/07L3O00000E6dDKUAZ.log"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
             self.assertTrue(1==2)
+
+    def test_log_file_to_file(self):
+        try:
+            res = InCli._main(["-logs","-inputfile",f"/Users/uormaechea/Documents/Dev/python/InCliLib/InCLipkg/incli/logs/07L3O00000E6dDKUAZ.log","-file"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
     def test_log_file_id(self):
         try:
             id = '07L0Q00000N5yVKUAZ'
@@ -354,10 +383,10 @@ class Test_Main(unittest.TestCase):
             print(traceback.format_exc())
             self.assertTrue(1==2)
 
-    def test_log_file_test(self):
+    def test_log_file_test_folder(self):
         try:
-            id = '07L3O00000Dwx2RUAR'
-            res = InCli._main(["-u","NOSDEV","-logs","-inputfile",f"/Users/uormaechea/Documents/Dev/python/InCliLib/InCLipkg/.InCli/logs/{id}.log"])
+            id = '07L3O00000EBQ6MUAX'
+            res = InCli._main(["-u","NOSDEV","-logs","-inputfile",f"/Users/uormaechea/Documents/Dev/python/InCliLib/InCLipkg/InCli/logs/{id}.log"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -417,7 +446,7 @@ class Test_Main(unittest.TestCase):
 
     def test_log_tail_loguser_guest_auto_delete(self):
         try:
-            InCli._main(["-u","NOSQSM","-logs","-tail","-loguser","username:onboarding@nosqms-parceiros.cs189.force.com","-auto","-deletelogs"])
+            InCli._main(["-u","demo240","-logs","-tail","-auto","-deletelogs"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -434,7 +463,16 @@ class Test_Main(unittest.TestCase):
     def test_log_tail_auto(self):
         try:
             #return
-            InCli._main(["-u","NOSDEV","-logs","-tail","-auto"])
+            InCli._main(["-u","devupg2","-logs","-tail","-auto"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)  
+
+    def test_log_tail_auto_delete(self):
+        try:
+            #return
+            InCli._main(["-u","DEVNOSCAT4","-logs","-tail","-auto","-deletelogs"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -443,7 +481,7 @@ class Test_Main(unittest.TestCase):
     def test_log_tail(self):
         try:
             #return
-            InCli._main(["-u","NOSQSM","-logs","-tail"])
+            InCli._main(["-u","DEVNOSCAT4","-logs","-tail"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -460,11 +498,21 @@ class Test_Main(unittest.TestCase):
     def test_log_tail_delete(self):
         try:
             #return
-            InCli._main(["-u","NOSQSM","-logs","-tail","-deletelogs"])
+            InCli._main(["-u","DEVNOSCAT4","-logs","-tail","-deletelogs"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
             self.assertTrue(1==2)     
+
+    def test_log_tail_delete_noScreen(self):
+        try:
+            #return
+            InCli._main(["-u","DEVNOSCAT4","-logs","-tail","-deletelogs","-noScreen"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)    
+
     def test_log_tail_where(self):
         try:
             return
@@ -539,7 +587,7 @@ class Test_Main(unittest.TestCase):
 
     def test_l(self):
         try:
-            InCli._main(["-u","NOSDEV","-l"])
+            InCli._main(["-u","NOSPRD","-l"])
             InCli._main(["-u","DEVNOSCAT2","-l"])
         except Exception as e:
             utils.printException(e)
@@ -556,4 +604,139 @@ class Test_Main(unittest.TestCase):
 
     def test_checkVersion(self):
         InCli.checkVersion()
-        print()
\ No newline at end of file
+        print()
+
+    def test_ipe(self):
+        try:
+            InCli._main(["-u","NOSPRD","-ipe"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_ipe_id(self):
+        try:
+            InCli._main(["-u","NOSPRD","-ipe","a6K7T000001dk4PUAQ"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_ipe_id_tofile(self):
+        try:
+            InCli._main(["-u","NOSDEV","-ipe","a6K3O000000FJ5WUAW","-file"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+    def test_ipe_last(self):
+        try:
+            InCli._main(["-u","NOSPRD","-ipe","-last","100"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_ipa(self):
+        try:
+            InCli._main(["-u","NOSDEV","-ipa"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_ipa_dates(self):
+        try:
+            InCli._main(["-u","NOSPRD","-ipa","-from","2023-03-09T09:19:31","-to","2023-03-11T13:55:00"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_ipa_around(self):
+        try:
+            InCli._main(["-u","NOSPRD","-ipa","-around","2023-03-28T16:25:00"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_ipa_contains(self):
+        try:
+            InCli._main(["-u","NOSPRD","-ipa","-around","2023-03-14T11:24:07","-contains","a3l7T000001RbofQAC,Marco"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)          
+
+    def test_ipa_limit(self):
+        try:
+            InCli._main(["-u","NOSPRD","-ipa","-limit","25"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_ipa_orderNumber(self):
+        try:
+            InCli._main(["-u","NOSPRD","-ipa","-orderNumber","00300083"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_ipa_id(self):
+        try:
+            InCli._main(["-u","NOSPRD","-ipa","a6a7T000002RNtlQAG"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_ipa_id_toFile(self):
+        try:
+            InCli._main(["-u","NOSQSM","-ipa","a2J7a000002UakuEAC","-file"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_vte(self):
+        try:
+            InCli._main(["-u","NOSPRD","-vte"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_vte_id(self):
+        try:
+            InCli._main(["-u","NOSDEV","-vte","a6a3O000000FFkYQAW"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_vte_id_file(self):
+        try:
+            InCli._main(["-u","NOSDEV","-vte","a6a3O000000FFkYQAW","-file"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_vte_wrong_command(self):
+        try:
+            InCli._main(["-u","NOSDEV","-xxx","a6a3O000000FFkYQAW","-file"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_vte_wrong_modifier(self):
+        try:
+            InCli._main(["-u","NOSDEV","-logs","a6a3O000000FFkYQAW","-dellogs"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
\ No newline at end of file
