@@ -50,7 +50,7 @@ class Test_Overrides(unittest.TestCase):
         call = restClient.callAPI(action)
         self.assertTrue(len(call)>0)    
         restClient.callSave("testFile")
-        lc = restClient.thread_lastCall()
+        lc = restClient.lastCall()
 
         self.assertTrue('responseFilePath' in lc)
         fileContent = jsonFile.read(lc['responseFilePath'])
@@ -105,7 +105,7 @@ class Test_Overrides(unittest.TestCase):
             action = '/services/data'
             call = restClient.callAPI(action)
             times= {
-                "elapsed":restClient.thread_lastCall_time()
+                "elapsed":restClient.lastCall()
             } 
             allTimes.append(times)
 
@@ -130,7 +130,7 @@ class Test_Overrides(unittest.TestCase):
         try:            
             restClient.init("uormaechea.devnoscat2@nos.pt")
             q = query.query(" select fields(all) from Order limit 1")
-            lc = restClient.thread_lastCall()
+            lc = restClient.lastCall()
             self.assertEqual(lc['status_code'],200)
             print(q)
 
