@@ -59,7 +59,7 @@ class Test_TraceFlag(unittest.TestCase):
 
     def test_query_who(self):
 
-        envs = ['DTI',"NOSDEV","NOSQSM"]
+        envs = ['DTI',"NOSDEV","NOSQSM","NOSPRD"]
         for env in envs:
             try:
                 restClient.init(env)
@@ -70,8 +70,6 @@ class Test_TraceFlag(unittest.TestCase):
                 res =tooling.query(q)
                 q1 = f"select username from User where Id='{res['records'][0]['TracedEntityId']}'"
                 res1 = query.query(q1)
-                for record in res1['records']:
-                    record.pop('attributes')
                 print()
                 print(f'Environment: {env}')
                 utils.printFormated(res1['records'])
@@ -79,3 +77,20 @@ class Test_TraceFlag(unittest.TestCase):
                 utils.printException(e)
 
         print()
+
+    def test_completions(self):
+        restClient.init('NOSDEV')
+        tooling.completions()
+
+    def test_aync(self):
+        restClient.init('NOSDEV')
+        tooling.executeAnonymous()
+
+    def test_limitUsageHistory(self):
+        restClient.init('NOSDEV')
+
+        q = "select fields(all) from NetworkPublicUsageDailyMetrics limit 10 "
+
+        res = tooling.query(q)
+
+        print()
